/*
************************************************************************
		an example of GKI implementation

************************************************************************
*/

class TTestGKI : public TGlobalKI
{
public:
	TFit bestfit;		//gobal best fitness

	TTestGKI(int period);	//constructor
	virtual ~TTestGKI();	//destructor

	//(de)serialization methods
	TTestGKI(BYTE** rawbytes,uint len,BOOL subclass=FALSE);		//see TObj
	virtual uint GetSerSize(BOOL subclass=FALSE); 			//see TObj
	virtual uint Serialize(BYTE **rawbytes,BOOL subclass=FALSE);	//see TObj

	//overriden methods
	virtual BOOL Init();
	virtual TKernelInfo* CreateKI(uint kid,BYTE** rawbytes,uint len);
	virtual void Compute();
};

TTestGKI::TTestGKI(int period) : TGlobalKI(KIG_TEST,period) 
{
	bestfit = 0;
}

TTestGKI::~TTestGKI() 
{
}

BOOL TTestGKI::Init()
{
	uint kis[1];	
	
	//just one LKI from each child process
	kis[0] = KIL_TEST;
	
	//init bag
	return InitBag(1,kis,lperiod);
}

TKernelInfo* TTestGKI::CreateKI(uint kid,BYTE** rawbytes,uint len)
{
	switch(kid)
	 {
		case KIL_TEST:	return new TTestLKI(rawbytes,len,TRUE);
		
		default:	NULL;
	 }	
}

void TTestGKI::Compute()
{
	uint i,j;
	TTestLKI *lki;
	TFit f;
	
	//search the bag and find best fitness
	lki = (TTestLKI*) bag->GetFirst();
	while(lki)
	{
		f = lki->best->GetFit();	//get fitness value of locally best gene
		if(f>=bestfit) bestfit = f;	//is better?

		lki = (TTestLKI*) bag->GetNext();
	}
}

uint TTestGKI::GetSerSize(BOOL subclass)
{
	uint sum = 0;
	
	//call predecestor
	sum += TGlobalKI::GetSerSize(TRUE);

	sum += GetSerSizeTFit(bestfit);

	return sum;
}

uint TTestGKI::Serialize(BYTE **rawbytes,BOOL subclass)
{
	uint sum = 0;
	//call predecestor
	sum += TGlobalKI::Serialize(rawbytes,TRUE);	
	sum += SerializeTFit(bestfit,rawbytes);

	return sum;
}

TTestGKI::TTestGKI(BYTE** rawbytes,uint len,BOOL subclass)
 : TGlobalKI(rawbytes,len,TRUE)		//call predecestor
{
	bestfit = DeserializeTFit(rawbytes);	
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//class TParentLayer is a layer of parent process overriden form TGlobalCollector class
TKernelInfo* TParentLayer::CreateKI(uint id,int period)
{
	switch(id)
	{
		case KIG_TEST: 	return new TTestGKI(period);
	}

	//call predecestor
	return TPredecestor::CreateKI(id,period);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//class TAppLayer is overriden from TAppCollector
TKernelInfo* TAppLayer::DeserializeKI(uint id,BYTE** rawbytes,uint len)
{
	switch(id)
	{
		case KIG_TEST:	return new TTestGKI(rawbytes,len);
	}

	//call predecestor
	return TAppCollector::DeserializeKI(id,rawbytes,len);
}
